/*
 *                (c) Copyright Macdonald Associates Ltd. 1983--2008
 *               All rights reserved. All trade marks respected.
 *
 * This software is the exclusive property of Macdonald Associates Ltd.
 * It is furnished under license and may only be used or copied in accordance
 * with the terms of such license.
 * This software is subject to change without notice and no information
 * contained in it should be construed as a commitment by Macdonald Associates.
 * Macdonald Associates cannot accept any responsibility, financial or
 * otherwise, for any consequences arising from the use of this software except
 * as otherwise stated in the terms of the license.
 *
 */
package com.macd.test.messagerouter;

import com.macd.test.Logger;
import java.io.Serializable;
import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import javax.ejb.EJBException;
import javax.ejb.Stateless;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSession;


/** This EJBean interfaces the MessageRouter used to route and send messages.
 *
 *  <br>
 *  @version <small><pre>
 *    SVN: repository            $URL: svn+ssh://svnserver/Macd-SVN-Repository/all-macd/branches/ALL_MACD_7_7_2_branch/src/com/macd/xConnect/ejbeans/messagerouter/MessageRouterBean.java $
 *    SVN: last committed   $Revision: 5819 $
 *    SVN: last committed       $Date: 2009-07-31 16:55:04 +0200 (Fri, 31 Jul 2009) $
 *    SVN: last committed     $Author: daniel $
 *  </pre></small>
 *
 *  @author (C) Macdonald Associates Ltd. All Rights Reserved.
 */
@Stateless(mappedName = MessageRouterInterface.GLOBAL_JNDI_NAME)
public class MessageRouterBean
        implements MessageRouterLocal, MessageRouterRemote {

    private final Logger logger = new Logger(MessageRouterBean.class, "" + this.hashCode());
    private MessageRouterPojo messageRouter;
    private QueueConnection queueConnection;
    private QueueSession queueSession;

    @Resource(name = "ConnectionFactory")
    private QueueConnectionFactory queueConnectionFactory;
    
    @PostConstruct
    public void postConstruct() {

        try {
            messageRouter = new MessageRouterPojo();

            // The connection and the session are created once
            // to avoid creating and closing the resources for each message to be sent.
            // Thus we avoid running out of resources on high messages volumes.
            queueConnection = queueConnectionFactory.createQueueConnection();
            queueSession = queueConnection.createQueueSession(false, javax.jms.Session.AUTO_ACKNOWLEDGE);
        } catch (Exception e) {
            logger.logError("ejbCreate failed: " + e.getMessage(), e);
            throw new EJBException(e);
        }
    }

    @PreDestroy
    public void preDestroy() {

        try {
            if (null != queueConnection) {
                queueConnection.close();
                queueConnection = null;
            }

        } catch (Exception e) {
            throw new EJBException(e.getMessage(), e);
        }
    }

    public void routeMessage(String myAddress,
            Object applicationMessage,
            int deliveryMode)
            throws MessageRouterException {

        logger.logInfo( "routeMessage: " + applicationMessage);
        try {
            messageRouter.routeMessage(queueSession, (Serializable) applicationMessage, deliveryMode, myAddress);
        } catch (Exception e) {
            throw new MessageRouterException(e.getMessage(), e);
        }
    }

    @Override
    public void sendMessage(Object inMessage,
            String targetAddress,
            int deliveryMode)
            throws MessageRouterException {

        logger.logInfo("sendMessage: " + inMessage);
        try {
            messageRouter.sendMessage(queueSession, (Serializable) inMessage, targetAddress, deliveryMode);
        } catch (Exception e) {
            throw new MessageRouterException(e.getMessage(), e);
        }
    }
}
