/*
 *                (c) Copyright Macdonald Associates Ltd. 1983--2008
 *               All rights reserved. All trade marks respected.
 *
 * This software is the exclusive property of Macdonald Associates Ltd.
 * It is furnished under license and may only be used or copied in accordance
 * with the terms of such license.
 * This software is subject to change without notice and no information
 * contained in it should be construed as a commitment by Macdonald Associates.
 * Macdonald Associates cannot accept any responsibility, financial or
 * otherwise, for any consequences arising from the use of this software except
 * as otherwise stated in the terms of the license.
 *
 */
package com.macd.test.messagerouter;

import com.macd.test.*;
import java.io.Serializable;

import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MessageProducer;
import javax.jms.ObjectMessage;
import javax.jms.Queue;
import javax.jms.Session;
import javax.naming.NamingException;

import javax.jms.Message;

/** Routes and sends messages to their original destination or to several destinations.
 *  <br>
 *  @version <small><pre>
 *    SVN: repository            $URL:$
 *    SVN: last committed   $Revision:$
 *    SVN: last committed       $Date:$
 *    SVN: last committed     $Author:$
 *  </small></pre>
 *
 *  @author (C) Macdonald Associates Ltd. All Rights Reserved.
 *
 */
public class MessageRouterPojo {

    private final static Logger logger = new Logger(MessageRouterPojo.class);

    public MessageRouterPojo() {
    }

    /**
     * routeMessage will route the given message to its target(s).
     */
    public void routeMessage(final Session queueSession,
            final Serializable applicationMessage,
            final int deliveryMode,
            final String myAddress)
            throws NamingException, JMSException {


        sendMessage(queueSession, applicationMessage, "", deliveryMode);

    }

    /** Send message to target. Adds JMSReplyTo if a destination is given.
     */
    public void sendMessage(final Session queueSession,
            final Serializable message,
            final String targetAddress,
            final int deliveryMode,
            final Destination replyTo)
            throws NamingException, JMSException {


        // set up queue sender
        final MessageProducer queueSender = queueSession.createProducer(queueSession.createQueue(targetAddress));

        // create jms message
        final ObjectMessage objectMessage = queueSession.createObjectMessage();
        objectMessage.setObject(message);
        objectMessage.setJMSDeliveryMode(deliveryMode);

        // set optional replyTo destination
        if (null != replyTo) {
            objectMessage.setJMSReplyTo(replyTo);
        }

        // send message
        queueSender.send(objectMessage);

        // cleanup
        queueSender.close();

        // XXX TestPerformance: logger.logInfo( "sendMessage: done" );
    }

    /**
     * Send message to target.
     */
    public void sendMessage(final Session queueSession,
            final Serializable applicationMessage,
            final String targetAddress,
            final int deliveryMode)
            throws NamingException, JMSException {

        sendMessage(queueSession, applicationMessage, targetAddress, deliveryMode, null);
    }
}
